unit LDN034u1;

{$mode objfpc}{$H+}

{Version: 10 Aug 18
 Started: 10 Aug 18

LDN034 is just my name for this little app: "34th Lazarus Demo, New series"
It was lifted wholesale from...
http://wiki.freepascal.org/Lazarus_on_Raspberry_Pi#Native_hardware_access
... being that page's example of how to read a PI GPIO pin, with just a
few tiny tweaks. (I replaced the CheckBox ("GPIO18CheckBox") with a label
I named "laStateOfInput18", because that made sense to me. The "Checkbox
solution" did WORK.

NOTA BENE: To create this app, you need to put a TApplicationProteries
on the form. It is a component like, say, a Timer component which creates
nothing that a user will see when the app is run. It is available from the
"Additional" tab of the Component Palette of the Lazarus IDE. You need to
the handler code that you see below, as a handler for the OnIdle event of
the ApplicationsProperties component.}

{Demo application for GPIO on Raspberry Pi}
{Inspired by the Python input/output demo application by Gareth Halfacree}
{written for the Raspberry Pi User Guide, ISBN 978-1-118-46446-5}

{This application reads the status of a push-button connecting GPIO pin 18 to
ground. Pin 18 is pulled high (3v3) through 1k if the button is not pressed.

"Pin 18"? There are at least two numbering schemes. This "18" is by the scheme
used to label Sparkfun's excellent
Pi Wedge:  (https://www.sparkfun.com/products/13717)

BE CAREFUL: Pin 18 should only be pulled up to 3v3. Don't use 5v by mistake.
Also... prudent to RUN program once BEFORE making connection to pin 18, so
that you are sure about what direction it is set for.

I'm not entirely clear about permissions.
I believe that you want to make the user which is being used to create
the code should be a member of at least the gpio groups. He/ she should
(perhaps) be part of the i2c and spo groups, as well.

With that fixed,
I don't think the app has to be run as the user "root".}


interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
      Unix, BaseUnix;//Note Unix and Basenix. These added by hand.

type

  { TLDN034f1 }

  TLDN034f1 = class(TForm)
    ApplicationProperties1: TApplicationProperties;
    Edit1: TEdit;
    GPIO18CheckBox: TCheckBox;
    Label1: TLabel;
    laStateOfInput18: TLabel;
    LogMemo: TMemo;
    Memo1: TMemo;
    procedure ApplicationProperties1Idle(Sender: TObject; var Done: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

const
  PIN_18: PChar = '18';
  IN_DIRECTION: PChar = 'in';

var
  LDN034f1: TLDN034f1;
  gReturnCode: longint; {stores the result of the IO operation}

implementation

{$R *.lfm}

{ TLDN034f1 }

procedure TLDN034f1.FormActivate(Sender: TObject);
var
  fileDesc: integer;
begin
  {GPIO18CheckBox wa in the original example. I've written it out
    of this version, but have left it in comments, to show what
    the old version was. Rem out the next line, if you want to use#
    GPIO18CheckBox.}

  top:=10;
  left:=10;

  laStateOfInput18.caption:='State of input unknown';

  GPIO18CheckBox.visible:=false;

  { Prepare SoC pin 18 (pin 12 on GPIO port) for access: }

  {The sleep(xx) statements in the following aren't quite right yet,
   but the code "works". (Sorry. If you can explain, I'd love to
   understand. The "sleeps" were critical in code to drive a GPIO
   pin as an output. Note that they are only necessary here. Perhaps
   how the CLEAN UP "goes" during FormClose has a bearing on what
   happens at next boot?}

  try
    fileDesc := fpopen('/sys/class/gpio/export', O_WrOnly);
    sleep(2000);//"Works" without this... but doesn't give right code
       //out of the following...
    gReturnCode := fpwrite(fileDesc, PIN_18[0], 2);//Should return "2"
    LogMemo.Lines.Add(IntToStr(gReturnCode));
  finally
    gReturnCode := fpclose(fileDesc);//Should return "0"
    LogMemo.Lines.Add(IntToStr(gReturnCode));
  end;
  { Set SoC pin 18 as input: }
  try
    fileDesc := fpopen('/sys/class/gpio/gpio18/direction', O_WrOnly);
    gReturnCode := fpwrite(fileDesc, IN_DIRECTION[0], 2);//Should return "2"
    LogMemo.Lines.Add(IntToStr(gReturnCode));
  finally
    gReturnCode := fpclose(fileDesc);//Should return "0"
    LogMemo.Lines.Add(IntToStr(gReturnCode));
  end;
  showmessage('So far, you SHOULD see 2, 0, 2, 0 in the memo... I think. But '+
     'I don''t always, and the app still works, when I don''t. When you acknowlege '+
     'this message, the memo should fill rapidly with 0, 1:x pairs, where '+
     'x is 1 if the input is high, 0 otherwise. Also the label will change.');
end;//FormActivate

procedure TLDN034f1.ApplicationProperties1Idle(Sender: TObject;
  var Done: Boolean);
var
  fileDesc: integer;
  buttonStatus: string[1] = '1';
begin
  try
    { Open SoC pin 18 (pin 12 on GPIO port) in read-only mode: }
    fileDesc := fpopen('/sys/class/gpio/gpio18/value', O_RdOnly);
    if fileDesc > 0 then
    begin
      { Read status of this pin (0: button pressed, 1: button released): }
      gReturnCode := fpread(fileDesc, buttonStatus[1], 1);
      LogMemo.Lines.Add(IntToStr(gReturnCode) + ': ' + buttonStatus);
      LogMemo.SelStart := Length(LogMemo.Lines.Text) - 1;
      if buttonStatus = '0' then begin
        //GPIO18CheckBox.Checked := true
        laStateOfInput18.font.color:=clGray;
        laStateOfInput18.caption:='Low';
        end//no ; here
      else begin
        //GPIO18CheckBox.Checked := false;
        laStateOfInput18.font.color:=clGreen;
        laStateOfInput18.caption:='High';
        end;
    end;
  finally
    { Close SoC pin 18 (pin 12 on GPIO port) }
    gReturnCode := fpclose(fileDesc);
    LogMemo.Lines.Add(IntToStr(gReturnCode));
    LogMemo.SelStart := Length(LogMemo.Lines.Text) - 1;
  end;
end;//ApplicationPropertiesIdle


procedure TLDN034f1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
var
  fileDesc: integer;
begin
  { Free SoC pin 18: }
  try
    fileDesc := fpopen('/sys/class/gpio/unexport', O_WrOnly);
    gReturnCode := fpwrite(fileDesc, PIN_18[0], 2);
    LogMemo.Lines.Add(IntToStr(gReturnCode));
  finally
    gReturnCode := fpclose(fileDesc);
    LogMemo.Lines.Add(IntToStr(gReturnCode));
  end;
end;//  FormClose

procedure TLDN034f1.FormCreate(Sender: TObject);
begin
  top:=10;
  left:=10;

  laStateOfInput18.caption:='State of input unknown';

  GPIO18CheckBox.visible:=false;
end; //Form Create


end.

